/*
 * MSP430-Launchpad
 * ================
 * Uwe Berger; 2013
 * 
 * zyklische Ausgabe des Messwertes 
 * des internen Temperatursensors
 * 
 * Have fun!
 * 
*/

#ifndef F_CPU
#error "no define F_CPU!"
#endif

#include "msp430.h"
#include "uart.h"

#define LED_RED   BIT0                // rote LED an PIN0
#define LED_GREEN BIT6                // gruene Led an PIN6

uint16_t temp, temp_old;
char buffer[6];
 
// *************************** 
// *************************** 
// *************************** 
int main(void) {
	WDTCTL  = WDTPW + WDTHOLD;				// Watchdog ausschalten
	BCSCTL1 = CALBC1_1MHZ;					// MCU-Taktrate
	DCOCTL  = CALDCO_1MHZ;
	P1DIR  = LED_RED | LED_GREEN;			// LED-Pins als Ausgaenge
	uart_init(9600);						// serielle Schnittstelle initialisieren
	// ADC konfigurieren
	ADC10CTL1 = INCH_10 + ADC10DIV_3;         						// Temp.Sensor ADC10CLK/4
	ADC10CTL0 = SREF_1 + ADC10SHT_3 + REFON + ADC10ON + ADC10IE;	// Referenz etc.
	__delay_cycles(1000);                     						// etwas Warten damit sich Referenz einstellt
	while (1) {                           
		ADC10CTL0 |= ENC + ADC10SC;             // Sampling and conversion start
		__bis_SR_register(CPUOFF + GIE);        // LPM0 mit Interrupts einschalten
		temp = ADC10MEM;
		P1OUT &= ~LED_GREEN;					// LEDs je nach Temp.diff setzen
		P1OUT &= ~LED_RED;
		if (temp<temp_old) {					
			P1OUT |= LED_GREEN;
		} else  if (temp>temp_old)
					P1OUT |= LED_RED;
		uart_printf("%i°C (ADC=%i)\n\r", (((temp - 630) * 761) / 1024), temp);	// Formel aus LP-Demo
		temp_old = temp;
		__delay_cycles(1000000);				// 1s warten
	}
	return 0;
}
 
// *************************** 
// ADC10 interrupt service routine
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR (void){			// feuert, wenn ADC-Wandlung fertig
	__bic_SR_register_on_exit(CPUOFF);      // MCU in aktiven Mode schalten
}
